#!/bin/bash
#
# This script aids the user to detect the name of a Linkat 4 Center server.
# It suggests the first nameserver entry of /etc/resolv.conf and let the user to write one.
# Then it checks the IP pinging it and does a nameserver reverse lookup to get the nameserver.
# If one is found, asks the user if is correct and echoes the server name.
# User can cancel the process and the script returns 1.
#
# Author: Jordi "Txor" Casas Ríos <jordi.casas@opentrends.net>
#

#############
# functions #
#############
# ask_and_verify_ip
# Asks the user for IP address and test it's connectivity with ping.
# Returns 0 on success, 1 otherwise.
# The parameter is the suggested IP address on the zenity dialog.
ask_and_verify_ip () {
	IP=$($ZENITY --entry --text="Si us plau introduïu l'adreça IP del Servidor de Centre." --entry-text=$1)
	if [ ! $? -eq 0 ] ; then
		return 1
	fi	
	ping -c 3 $IP | zenity --progress --pulsate --auto-close --text="Fent ping a l'adreça IP $IP ..." >/dev/null 2>&1
	if [ ${PIPESTATUS[0]} -eq 0 ] ; then
		echo $IP
		return 0
	fi
	zenity --error --title="Error de connexió" --text="No s'ha pogut contactar amb l'adreça IP $IP" --timeout=3 >/dev/null 2>&1
	return 1
}

# check_and_get_nameserver
# Makes a reverse nameserver lookup with the argument as the IP address.
# Returns 0 and echoes the name on success, returns 1 otherwise.
check_and_get_nameserver () {
	DIG=$(dig @$1 -x $1 | grep PTR | grep -v DiG | grep -v ";" | sed -e 's:[^ ]* *[^ ]* *IN[ |\t]*PTR[ |\t]*\(.*\)\.:\1:' | tee >(zenity --progress --pulsate --auto-close --text="Buscant el nom del Servidor de centre ...") ; exit ${PIPESTATUS[0]})
        if [ $? -eq 0 ] ; then
		echo $DIG
                return 0
	else
		return 1
        fi
}

#####################
# Dynamic variables #
#####################
ZENITY=$(which zenity)
IPSERVIDOR=$(cat /etc/resolv.conf | grep nameserver | grep -v "^#" | head -1 | cut -d " " -f 2 2>&1 )

#############
# Main code #
#############
# While there's no valid nameserver nor canceled by the user, continue trying.
NSLOOKUP=1
while [ $NSLOOKUP -eq 1 ] ; do 
	$ZENITY --question --title="Configuració del DNS" --text="Es procedirà a detectar nom del Servidor de centre. Voleu continuar?"
	if [ ! $? -eq 0 ] ; then
		exit 1
	fi
	# While there's not responding server continue asking user for server IP.
	PING_SERV=1
	while [ $PING_SERV -eq 1 ] ; do 
		CURR_IP=$(ask_and_verify_ip $IPSERVIDOR)
		if [ $? -eq 0 ] ; then
			PING_SERV=0
		fi
	done
	# We have a responding IP, let's see if it has a DNS service.
	NAME=$(check_and_get_nameserver $CURR_IP)
	if [ $? -eq 0 ] ; then
		if [ -z $NAME ] ; then
			$ZENITY --info --title="Error" --text="No s'ha trobat un nom de servidor." --timeout=20
		else
			$ZENITY --question --title="Servidor trobat!" --text="Voleu utilitzar $NAME com a DNS?"
			if [ $? -eq 0 ] ; then
				NSLOOKUP=0
			fi
		fi
	else
		$ZENITY --info --title="Error" --text="No s'ha trobat un nom de servidor." --timeout=20
	fi
done
echo $CURR_IP $NAME
